/* Generate a blanket flapping in a breeze blowing toward the origin. */
/* Chris Walquist, February 26, 1991 */

#include "df.h"
#include "vg.h"

# define PI 3.14159276
# define SQ_SIZE 30
# define MAX_VAL (2.0 * PI)
# define FRAMES 20
# define FRAME_INC (MAX_VAL/FRAMES)
# define XY_INC MAX_VAL/SQ_SIZE
# define VFILENAME "sin.hdf"
# define NPOINTS (SQ_SIZE * SQ_SIZE)

float PZ[FRAMES][NPOINTS];
float PX[NPOINTS];
float PY[NPOINTS];
int plist4[FRAMES*NPOINTS][4];
int plist_cnt=0;

main(argc,argv) int argc; char **argv; {

   /* build the data set. */
   build_points();
   /* build the polygon list. */
   build_polygons();
   /* Write to a vset file. */
   write_data();

}


build_points() {
   int x,y, frame, index;
   double sin();
   float offset;

   /* Add x and y points. */
   for(x=0;x< SQ_SIZE; x++) {
      for(y=0;y<SQ_SIZE; y++) {
	 index=(x * SQ_SIZE) + y;
	 PX[index] = x * (float) XY_INC;
	 PY[index] = y * (float) XY_INC;
      }
   }

   /* Add Z points. */
   for (frame = 0; frame < FRAMES; frame++) {
      float xval,yval;

      offset = frame * FRAME_INC;
      for(x=0;x< SQ_SIZE; x++) {
	 for(y=0;y<SQ_SIZE; y++) {
	    index=(SQ_SIZE*x) + y;
	    PZ[frame][index] = sin(PX[index]+offset) - sin(PY[index]+offset+PI/2);
	 }
      }
   }
}


build_polygons() {
   int x,y;

   for (x=0;x<SQ_SIZE-1;x++) {
      for (y=0;y<SQ_SIZE-1;y++) {
	 int p[4];

	 p[0] = calc_pnt_index(x,y);
	 p[1] = calc_pnt_index(x,y+1);
	 p[2] = calc_pnt_index(x+1,y+1);
	 p[3] = calc_pnt_index(x+1,y);

	 connect4(p[0],p[1],p[2],p[3]);
      }
   }
}


int
calc_pnt_index(x,y) int x,y; {
   return ((x * SQ_SIZE) + y);
}


connect4(p0,p1,p2,p3) int p0,p1,p2,p3; {
   plist4[plist_cnt][0] = p0+1;
   plist4[plist_cnt][1] = p1+1;
   plist4[plist_cnt][2] = p2+1;
   plist4[plist_cnt][3] = p3+1;
   plist_cnt++;
   return plist_cnt-1;
}


write_data() {
   DF *f;
   VGROUP *vg, *top_vg, *Vattach();
   VDATA *x_vsptr, *r_vsptr, *y_vsptr, *z_vsptr,
    *plist_vsptr, *data_vsptr, *VSattaach();
   int frame;
   char group_name[100];

   f = DFopen(VFILENAME,DFACC_CREATE,0);
   printf("Opening vset file \"%s\"\n",VFILENAME);

   for (frame = 0; frame < FRAMES; frame++) {
      /* Create a new vgroup. */
      vg = (VGROUP *) Vattach(f,-1,"w");
      sprintf(group_name,"group%03d",frame+1);
      Vsetname(vg,group_name);

      if (frame==0) {
	 /* Add the x-y points and the polygon list on the first go-around. */

	 /* Add the vdata for points to the vset file, and add to the vgroup. */
	 x_vsptr = (VDATA *) VSattach(f,-1,"w");
	 VSsetname(x_vsptr,"px");
	 VSsetfields(x_vsptr,"px");
	 VSwrite(x_vsptr,PX,NPOINTS,NO_INTERLACE);
	 Vinsert(vg,x_vsptr);
	 VSdetach(x_vsptr);

	 y_vsptr = (VDATA *) VSattach(f,-1,"w");
	 VSsetname(y_vsptr,"py");
	 VSsetfields(y_vsptr,"py");
	 VSwrite(y_vsptr,PY,NPOINTS,NO_INTERLACE);
	 Vinsert(vg,y_vsptr);
	 VSdetach(y_vsptr);

	 /* add the polygon list to the vset file. */
	 plist_vsptr = (VDATA *) VSattach(f, -1, "w");
	 VSfdefine(plist_vsptr, "plist4", LOCAL_INTTYPE, 4);
	 VSsetname(plist_vsptr, "plist4");
	 VSsetfields(plist_vsptr, "plist4");
	 VSwrite(plist_vsptr, plist4, plist_cnt, FULL_INTERLACE);
	 Vinsert(vg, plist_vsptr);
	 VSdetach(plist_vsptr);
      } 

      /* Add the changing parts of the data */
      z_vsptr = (VDATA *) VSattach(f,-1,"w");
      VSsetname(z_vsptr,"pz");
      VSsetfields(z_vsptr,"pz");
      VSwrite(z_vsptr,PZ[frame],NPOINTS,NO_INTERLACE);
      Vinsert(vg,z_vsptr);
      VSdetach(z_vsptr);

      Vdetach(vg);
   }
   DFclose(f);
}
